"""
pyGoogleTranslate caching system.

INCLUDES:
- import_cache()
- export_cache()
- search_translation_cache()
- add_translation_cache()

© Anime no Sekai - 2020
"""

import os
from lifeeasy import read_file, write_file, today, current_time, working_dir, find_inside_file
from .language_code import verify_language_code

translation_caches_path = os.path.dirname(os.path.abspath(__file__)) + '/translation_caches/'
translation_caches_name = 'translation_caches.animenosekai_caches'

def import_cache(cache_file_location):
    """
    Import a .animenosekai_caches translation cache file.

    © Anime no Sekai - 2020
    """
    current_cache_file_content = read_file(translation_caches_path + translation_caches_name).split('\n')
    new_cache_file_content = read_file(cache_file_location).split('\n')
    write_file(translation_caches_name, f'\n\n[IMPORTED {today()} {current_time()}]\n', translation_caches_path, append=True)
    for line in new_cache_file_content:
        if not line in current_cache_file_content:
            write_file(translation_caches_name, line, translation_caches_path, append=True)
    
def export_cache(destination=working_dir):
    """
    Export the cache file to be imported later (after an update for example).

    © Anime no Sekai - 2020
    """
    write_file('pyGoogleTranslateCacheExport.animenosekai_caches', read_file(translation_caches_path + translation_caches_name), destination=destination)

def search_translation_cache(source_language, destination_language, source):
    """
    Searches a translation through the caches.
    """
    line_start = f'[sl={verify_language_code(source_language)}]｜[dl={verify_language_code(destination_language)}]｜[source={source}]'
    search_result = find_inside_file(translation_caches_path + translation_caches_name, line_start)
    if not search_result == {}:
        return {'result': search_result['line'].split('｜')[3][8:][:-2], 'line_number': str(search_result['line_number'])}
    else:
        line_start = f'[sl={verify_language_code(destination_language)}]｜[dl={verify_language_code(source_language)}]'
        search_result = find_inside_file(translation_caches_path + translation_caches_name, line_start, whole_document=True)
        if not search_result == {}:
            found = False
            for result in search_result:
                if result['line'].split('｜')[3][8:][:-2] == source:
                    found = True
                    return {'result': result['line'].split('｜')[2][8:][:-1], 'line_number': str(result['line_number'])}
            if not found:
                return None
        else:
            return None

def add_translation_cache(source_language, destination_language, source, result):
    """
    Adds a translation to the cache file.
    """   
    line = f'[sl={verify_language_code(source_language)}]｜[dl={verify_language_code(destination_language)}]｜[source={source}]｜[result={result}]\n'
    write_file(translation_caches_name, line, translation_caches_path, append=True)



"""
Used to verify language codes/translate from language name to language code (i.e french --> fr)

© Anime no Sekai - 2020
"""

from . import google_translate_data

class LanguageCodeError(Exception):
    """
    When the language code is incorrect.
    """
    def __init__(self, msg=None):
        self.msg = msg 
    def __str__(self):
        exception_msg = f"\n\n⚠️ ⚠️ ⚠️\n{self.msg}\n"
        return exception_msg

def verify_language_code(language):
    """
    Verifies and gets the correct language code.
    """
    if language.lower() == 'zh-cn':
        return 'zh-CN'
    if language.lower() == 'zh-tw':
        return 'zh-TW'
    if language.lower() in google_translate_data.language_code_data():
        return language.lower()
    else:
        try:
            result = google_translate_data.language_name_to_code()[language.lower()]
            return result
        except:
            try:
                result = google_translate_data.international_language_name_to_code()[language.lower()]
                return result
            except:
                raise LanguageCodeError(f'{language} is not a correct language code.')



"""
Set of internal and/or less used functions for pyGoogleTranslate.

© Anime no Sekai - 2020
"""

from . import caching
from . import domain
from . import google_translate_data
from . import language_code


"""
Setting the Google Translate domain name used by pyGoogleTranslate.

© Anime no Sekai - 2020
"""

from . import google_translate_data

gt_domain = 'translate.google.com'

class GoogleTranslateDomainError(Exception):
    """
    When the domain is incorrect.
    """
    def __init__(self, msg=None):
        self.msg = msg 
    def __str__(self):
        exception_msg = f"\n\n⚠️ ⚠️ ⚠️\n{self.msg}\n"
        return exception_msg

def google_translate_domain(domain='translate.google.com'):
    """
    Changes the used google translate domain name (i.e translate.google.com) to the given domain (i.e translate.google.co.jp)
    """
    global gt_domain
    clean_domain = domain.lower().replace('https://', '').replace('http://', '').replace('www.', '').split('/')[0]
    if clean_domain in google_translate_data.google_translate_domains_data():
        gt_domain = clean_domain
        return gt_domain
    else:
        raise GoogleTranslateDomainError(f'{clean_domain} is not a correct Google Translate domain.\n {gt_domain} will be used instead.')



from setuptools import setup
from os import path
with open(path.join(path.abspath(path.dirname(__file__)), 'README.md'), encoding='utf-8') as f:
    readme_description = f.read()
setup(
name = "pyGoogleTranslate",
packages = ["pyGoogleTranslate"],
version = "3.5.2",
license = "MIT",
description = "A python module which lets you use Google Translate (translation, transliteration, defintion, language detection, etc.) by parsing the website.",
author = "Anime no Sekai",
author_email = "niichannomail@gmail.com",
url = "https://github.com/Animenosekai/pyGoogleTranslate",
download_url = "https://github.com/Animenosekai/pyGoogleTranslate/archive/v3.5.2.tar.gz",
keywords = ['translate', 'translation', 'google_translate', 'google', 'selenium', 'animenosekai'],
install_requires = ['selenium', 'lifeeasy', 'psutil'],
classifiers = ['Development Status :: 5 - Production/Stable', 'License :: OSI Approved :: MIT License', 'Programming Language :: Python :: 3', 'Programming Language :: Python :: 3.4', 'Programming Language :: Python :: 3.5', 'Programming Language :: Python :: 3.6', 'Programming Language :: Python :: 3.7', 'Programming Language :: Python :: 3.8'],
long_description = readme_description,
long_description_content_type = "text/markdown",
include_package_data=True
)
